#!/usr/bin/perl -w
#
# file    : download-qd-files.pl
# changed : 2005-02-18
# author  : Bas Couwenberg <sebastic@xs4all.nl>
#

use strict;
use LWP::UserAgent;
use File::Basename;
use Date::Calc;
use Getopt::Mixed "nextOption";

my ($year, $month, $day, $hour, $min, $sec) = Date::Calc::Today_and_Now();

$month = '0'.$month if(length($month) == 1);
$day   = '0'.$day   if(length($day)   == 1);
$hour  = '0'.$hour  if(length($hour)  == 1);
$min   = '0'.$min   if(length($min)   == 1);
$sec   = '0'.$sec   if(length($sec)   == 1);

my $timestamp = "$year-$month-$day $hour:$min:$sec";

my $self    = 'download-qd-files.pl';
my $version = '0.1';
my $wwwroot = '/var/www/folding/mod_perl/qd-files/';

# default configuration
my %cfg = (
		self      => $self,
		version   => $version,
		url 	  => 'http://www.boston.quik.com/rph/',
		wwwroot   => $wwwroot,
		useragent => "$self/$version",
		verbose   => 0
	);
	
Getopt::Mixed::init("url=s wwwroot=s useragent=s verbose help");
while (my ($option, $value) = nextOption()) {
	if($option eq 'help') {
		print_usage();
		Getopt::Mixed::cleanup();
		exit;
	}
	
	$cfg{wwwroot}   = $value if($option eq 'wwwroot');
	$cfg{url}       = $value if($option eq 'url');
	$cfg{useragent} = $value if($option eq 'useragent');
	$cfg{verbose}   = 1      if($option eq 'verbose');
}
Getopt::Mixed::cleanup();

$cfg{'qdinfo-file'}           = $cfg{wwwroot}.'qdinfo.dat';
$cfg{'qd-bin'}                = $cfg{wwwroot}.'qd';
$cfg{'qd-windows-bin'}	      = $cfg{wwwroot}.'qd.exe';

$cfg{'qdinfo-file-update'}    = $cfg{'qdinfo-file'}.'.lastupdate';
$cfg{'qd-bin-update'}         = $cfg{'qd-bin'}.'.lastupdate';
$cfg{'qd-windows-bin-update'} = $cfg{'qd-windows-bin'}.'.lastupdate';

if($cfg{verbose}) {
	print basename($0)." v$cfg{version} started.\n\n";
}

my $error = 0;
my $msg   = '';

my $qdinfo_url = $cfg{'url'}.'qdinfo.dat';
if(valid_url($qdinfo_url)) {	
	if(my $qdinfo_data = fetch_url($qdinfo_url)) {
		if(-w $cfg{'qdinfo-file'} && -r $cfg{'qdinfo-file'}) {
			# check if version of just downloaded qdinfo.dat is newer than the one on disc
		
			my %qdinfo = parse_qdinfo($cfg{'qdinfo-file'});
					
			(my $qdinfo_oldversion = $qdinfo{da}) =~ s/^(\d+ \w+ \d+).*?$/$1/;
			 my $qdinfo_newversion = $qdinfo_oldversion;

			#da 17 February 2005 (rph)
			if($qdinfo_data =~ /da (\d+ \w+ \d+)/) {
				$qdinfo_newversion = $1;
			}
			
			if($cfg{verbose}) {
				print "Current qdinfo version: $qdinfo_oldversion\n";
				print "Latest  qdinfo version: $qdinfo_newversion\n\n";
			}
			
			if($qdinfo_oldversion eq $qdinfo_newversion) {
				$msg .= "You already have the latest version\n";
			}
			else {
				save_file($cfg{'qdinfo-file'}, $qdinfo_data);
						
				open(F, "> $cfg{'qdinfo-file-update'}") || die "Cannot open file ($!)";
				print F "$timestamp\n";
				close F;

				$msg .= "Successfully updated qdinfo.dat  ($qdinfo_newversion fr $qdinfo{fr})\n";

				# check if version of qdinfo.dat is newer than the version of qd already on disc
				
				#qd released 6 February 2005 (fr 028)
				my $qd_version = `"$cfg{'qd-bin'}" -v`;
				if($qd_version =~ s/^qd released //) {}
					
				(my $qd_fr     = $qd_version)  =~ s/^.*fr (\d+).*$/$1/;
					
				#6 February 2005 (fr 028)
				if($qd_version =~ s/^(\d+ \w+ \d+) .*?$/$1/) {}

				my $qdinfo_version  = $qdinfo_newversion;
				my $qdinfo_fr       = $qdinfo{fr};

				chomp($qd_version, $qd_fr);
				
				if($cfg{verbose}) {
					print "Current functional revision: $qd_fr\n";
					print "Latest  functional revision: $qdinfo_fr\n\n";
				}
					
				if($qd_version ne $qdinfo_version) {
					# file has been changed
					# point table is updated
							
					if($qd_fr ne $qdinfo_fr) {
						# new functional revision, 
						# qd has changed more than just including the latest point table
	
						my $qd_url     = $cfg{'url'}.'qd';
							
						if(valid_url($qd_url)) {
							if(my $qd_data = fetch_url($qd_url)) {
								if(!-e $cfg{'qd-bin'} || -w $cfg{'qd-bin'}) {
									save_file($cfg{'qd-bin'}, $qd_data);
											
									open(F, "> $cfg{'qd-bin-update'}") || die "Cannot open file ($!)";
									print F "$timestamp\n";
									close F;
								}
								else {
									$error = 1;
									$msg  .= "Cannot write file! ($cfg{'qd-bin'})\n";
								}
							}
							else {
								$error = 1;
								$msg  .= "Failed to fetch URL! ($qd_url)\n";
							}
						}
						else {
							$error = 1;
							$msg  .= "Invalid URL or HTTP error! ($qd_url)\n";
						}
						
						my $qdexe_url = $cfg{'url'}.'qd.exe';
					
						if(valid_url($qdexe_url)) {
							if(my $qdexe_data = fetch_url($qdexe_url)) {
								if(!-e $cfg{'qd-windows-bin'} || -w $cfg{'qd-windows-bin'}) {
									save_file($cfg{'qd-windows-bin'}, $qdexe_data);
									
									open(F, "> $cfg{'qd-windows-bin-update'}") || die "Cannot open file ($!)";
									print F "$timestamp\n";
									close F;
								}
								else {
									$error = 1;
									$msg  .= "Cannot write file! ($cfg{'qd-windows-bin'})\n";
								}
							}
							else {
								$error = 1;
								$msg  .= "Failed to fetch URL! ($qdexe_url)\n";
							}
						}
						else {
							$error = 1;
							$msg  .= "Invalid URL or HTTP error! ($qdexe_url)\n";
						}
									
						(my $qd_version = `"$cfg{'qd-bin'}" -v`) =~ s/^qd released //;
						    $qd_version =~ s/(\(|\))//g;
						chomp($qd_version);
		
						$msg .= "Successfully updated qd & qd.exe ($qd_version)\n";
					}
					else {
						$msg .= "You already have the latest functional revision of qd and qd.exe\n";
					}
				}
			}
		}
		else {
			$error = 1;
			if(!-w $cfg{'qdinfo-file'}) {
				$msg  .= "Cannot write file! ($cfg{'qdinfo-file'})\n";
			}
			if(!-r $cfg{'qdinfo-file'}) {
				$msg  .= "Cannot read file! ($cfg{'qdinfo-file'})\n";
			}
		}
	}
	else {
		$error = 1;
		$msg  .= "Failed to fetch URL! ($qdinfo_url)\n";
	}
}
else {
	$error = 1;
	$msg  .= "Invalid URL or HTTP error! ($qdinfo_url)\n";
}

if($error) {
	print "One or more error were encounterd:\n$msg\n";
}
else {
	if($cfg{verbose}) {
		print "$msg\n";
	}
}


####################[ Subroutines ]#################### 

sub save_file {
	my $filename = $_[0];
	my $content  = $_[1];

	print "Saving file ($filename)\n\n" if($cfg{verbose});
	
	if(!-e $filename || -w $filename) {
		open(F, "> $filename") || die "Cannot open file ($!)";
		print F $content;
		close F;
		
		return 1;
	}
	else {
		return undef;
	}
}

sub fetch_url {
	my $url = $_[0];
	
	print "Retrieving URL ($url)\n" if($cfg{verbose});
	
	my $ua  = LWP::UserAgent->new();
	   $ua->agent($cfg{useragent});
	my $req = HTTP::Request->new(GET => $url);
	my $res = $ua->request($req);
	
	if($res->is_success) {
		return $res->content;
	}
	else {
		return undef;
	}
}

sub valid_url {
	my $url = $_[0];
	
	print "Verifying URL ($url)\n" if($cfg{verbose});
	
	my $ua  = LWP::UserAgent->new();
	   $ua->agent($cfg{useragent});
	my $req = HTTP::Request->new(HEAD => $url);
	my $res = $ua->request($req);
	
	if($res->is_success()) {
		return 1;
	}
	elsif($res->is_error()) {
		return 0;
	}
	else {
		return 0;
	}
}

sub parse_qdinfo {
	my $file = $_[0];
	
	my %info;
	if(-r $file) {
		open(F, $file) || die "Cannot open file ($!)";
		while(<F>) {
			chomp;
			
			if(s/\r//) {}

			#fr 028
			if(/^(fr) (\d+)/) {
				$info{$1} = $2;
			}
			
			#da 17 February 2005 (rph)
			if(/^(da) (\d+ \w+ \d+ \(.*?\))/) {
				$info{$1} = $2;
			}

			#pg 09A7B4F9
			if(/^(pg) (\w+)/) {
				$info{$1} = $2;
			}

			#pt  101  7400   111   500   112   500   113   500   114   500
			#
			#not parsed at the moment
			
			#ph 1109  1100 0817020C
			#ph 1110 12800 0818619C  1110  7090 0818538C
			#ph 1113  2400 0881DEC4  1113 24000 087D79C4  1113 20000 08784FE4
			#
			#not parsed at the moment
		}
		close F;
	}

	return %info;
}

sub print_usage {
	my $verbose = $cfg{verbose} ? 'on' : 'off';
	
	print "$cfg{self} v$cfg{version}\n\n";
	print "Usage: ".basename($0)." [OPTIONS]\n\n";
	print "Options:\n";
	print "--url <url>          URL to qd website            (default: $cfg{'url'})\n";
	print "--wwwroot <dir>      Directory to store qd files  (default: $cfg{'wwwroot'})\n";
	print "--useragent <name>   User-Agent string            (default: $cfg{'useragent'})\n";
	print "--verbose            Verbose (debug) output       (default: $verbose)\n";
	print "--help               Print this information\n";
}
